@@ -270,5 +270,5 @@ hildon_desktop_item_find_by_id (HildonDesktopItem *item,
   if (!item->id)
     return -1;
 
-  return g_str_equal (item->id, id);
+  return !g_str_equal (item->id, id);
 }
